package scripts;

import com.stencyl.graphics.G;
import com.stencyl.graphics.BitmapWrapper;

import com.stencyl.behavior.Script;
import com.stencyl.behavior.Script.*;
import com.stencyl.behavior.ActorScript;
import com.stencyl.behavior.SceneScript;
import com.stencyl.behavior.TimedTask;

import com.stencyl.models.Actor;
import com.stencyl.models.GameModel;
import com.stencyl.models.actor.Animation;
import com.stencyl.models.actor.ActorType;
import com.stencyl.models.actor.Collision;
import com.stencyl.models.actor.Group;
import com.stencyl.models.Scene;
import com.stencyl.models.Sound;
import com.stencyl.models.Region;
import com.stencyl.models.Font;

import com.stencyl.Engine;
import com.stencyl.Input;
import com.stencyl.Key;
import com.stencyl.utils.Utils;

import openfl.ui.Mouse;
import openfl.display.Graphics;
import openfl.display.BlendMode;
import openfl.display.BitmapData;
import openfl.display.Bitmap;
import openfl.events.Event;
import openfl.events.KeyboardEvent;
import openfl.events.TouchEvent;
import openfl.net.URLLoader;

import box2D.common.math.B2Vec2;
import box2D.dynamics.B2Body;
import box2D.dynamics.B2Fixture;
import box2D.dynamics.joints.B2Joint;

import motion.Actuate;
import motion.easing.Back;
import motion.easing.Cubic;
import motion.easing.Elastic;
import motion.easing.Expo;
import motion.easing.Linear;
import motion.easing.Quad;
import motion.easing.Quart;
import motion.easing.Quint;
import motion.easing.Sine;

import com.stencyl.graphics.shaders.BasicShader;
import com.stencyl.graphics.shaders.GrayscaleShader;
import com.stencyl.graphics.shaders.SepiaShader;
import com.stencyl.graphics.shaders.InvertShader;
import com.stencyl.graphics.shaders.GrainShader;
import com.stencyl.graphics.shaders.ExternalShader;
import com.stencyl.graphics.shaders.InlineShader;
import com.stencyl.graphics.shaders.BlurShader;
import com.stencyl.graphics.shaders.SharpenShader;
import com.stencyl.graphics.shaders.ScanlineShader;
import com.stencyl.graphics.shaders.CSBShader;
import com.stencyl.graphics.shaders.HueShader;
import com.stencyl.graphics.shaders.TintShader;
import com.stencyl.graphics.shaders.BloomShader;



class Design_2_2_FollowPoints extends ActorScript
{
	public var _XPoint:Float;
	public var _YPoint:Float;
	public var _Follow:Bool;
	public var _PointReached:Bool;
	public var _WaitTime:Float;
	public var _FloatAfterPoints:Bool;
	public var _Acceleration:Float;
	public var _TopSpeed:Float;
	public var _PointIndex:Float;
	public var _UsedList:Array<Dynamic>;
	public var _Slowdown:Float;
	public var _StopwhenBattling:Bool;
	public var _LoopMode:String;
	public var _Reverse:Bool;
	public var _Debug:Bool;
	public var _DebugColor:Int;
	public var _DebugColor2:Int;
	public var _HeighttoJump:Float;
	public var _CreateRandomPoint:Bool;
	public var _RandomPointRadius:Float;
	public var _RandomAngle:Float;
	public var _PointsList:Array<Dynamic>;
	public var _JumpRange:Float;
	public var _XRange:Float;
	public var _YRange:Float;
	public var _JumpForce:Float;
	public var _JumpSlowdown:Float;
	public var _Jumped:Bool;
	
	/* ========================= Custom Event ========================= */
	public function _customEvent_Jump():Void
	{
		if(!(_Jumped))
		{
			_Jumped = true;
			propertyChanged("_Jumped", _Jumped);
			actor.setYVelocity(-(_JumpForce));
		}
	}
	public function _customEvent_FollowPoint():Void
	{
		_Follow = true;
		propertyChanged("_Follow", _Follow);
	}
	public function _customEvent_StopFollowPoint():Void
	{
		_Follow = false;
		propertyChanged("_Follow", _Follow);
		actor.setXVelocity((actor.getXVelocity() * _Slowdown));
		actor.setYVelocity((actor.getYVelocity() * _Slowdown));
	}
	
	/* ========================= Custom Block ========================= */
	public function _customBlock_GoToPoint(__XPoint:Float, __YPoint:Float, __Run:Bool):Void
	{
		var __Self:Actor = actor;
		_XPoint = asNumber(__XPoint);
		propertyChanged("_XPoint", _XPoint);
		_YPoint = asNumber(__YPoint);
		propertyChanged("_YPoint", _YPoint);
		Utils.clear(_UsedList);
		_UsedList = new Array<Dynamic>();
		propertyChanged("_UsedList", _UsedList);
		_customEvent_FollowPoint();
	}
	
	/* ========================= Custom Block ========================= */
	public function _customBlock_SetPath(__PathList:Array<Dynamic>):Void
	{
		var __Self:Actor = actor;
		if(!((__PathList.length == 0)))
		{
			_UsedList = __PathList;
			propertyChanged("_UsedList", _UsedList);
			_PointIndex = asNumber(0);
			propertyChanged("_PointIndex", _PointIndex);
			_XPoint = asNumber(asNumber(("" + _UsedList[Std.int(_PointIndex)]).split(",")[Std.int(0)]));
			propertyChanged("_XPoint", _XPoint);
			_YPoint = asNumber(asNumber(("" + _UsedList[Std.int(_PointIndex)]).split(",")[Std.int(1)]));
			propertyChanged("_YPoint", _YPoint);
			_customEvent_FollowPoint();
		}
		else
		{
			_customEvent_StopFollowPoint();
		}
	}
	
	
	public function new(dummy:Int, actor:Actor, dummy2:Engine)
	{
		super(actor);
		nameMap.set("Actor", "actor");
		nameMap.set("X Point", "_XPoint");
		_XPoint = 0.0;
		nameMap.set("Y Point", "_YPoint");
		_YPoint = 0.0;
		nameMap.set("Follow", "_Follow");
		_Follow = true;
		nameMap.set("Point Reached", "_PointReached");
		_PointReached = false;
		nameMap.set("Wait Time", "_WaitTime");
		_WaitTime = 0.0;
		nameMap.set("Float After Points", "_FloatAfterPoints");
		_FloatAfterPoints = false;
		nameMap.set("Acceleration", "_Acceleration");
		_Acceleration = 5.0;
		nameMap.set("Top Speed", "_TopSpeed");
		_TopSpeed = 10.0;
		nameMap.set("Point Index", "_PointIndex");
		_PointIndex = 0.0;
		nameMap.set("Used List", "_UsedList");
		_UsedList = [];
		nameMap.set("Slowdown", "_Slowdown");
		_Slowdown = 0.5;
		nameMap.set("Stop when Battling?", "_StopwhenBattling");
		_StopwhenBattling = false;
		nameMap.set("Loop Mode", "_LoopMode");
		_LoopMode = "";
		nameMap.set("Reverse", "_Reverse");
		_Reverse = false;
		nameMap.set("Debug?", "_Debug");
		_Debug = false;
		nameMap.set("Debug Color", "_DebugColor");
		_DebugColor = Utils.getColorRGB(0,128,128);
		nameMap.set("Debug Color 2", "_DebugColor2");
		_DebugColor2 = Utils.getColorRGB(0,0,0);
		nameMap.set("Height to Jump", "_HeighttoJump");
		_HeighttoJump = 10.0;
		nameMap.set("Create Random Point?", "_CreateRandomPoint");
		_CreateRandomPoint = false;
		nameMap.set("Random Point Radius", "_RandomPointRadius");
		_RandomPointRadius = 5.0;
		nameMap.set("Random Angle", "_RandomAngle");
		_RandomAngle = 0.0;
		nameMap.set("Points List", "_PointsList");
		_PointsList = [];
		nameMap.set("Jump Range", "_JumpRange");
		_JumpRange = 50.0;
		nameMap.set("X Range", "_XRange");
		_XRange = 3.0;
		nameMap.set("Y Range", "_YRange");
		_YRange = 5.0;
		nameMap.set("Jump Force", "_JumpForce");
		_JumpForce = 18.0;
		nameMap.set("Jump Slowdown", "_JumpSlowdown");
		_JumpSlowdown = 0.85;
		nameMap.set("Jumped", "_Jumped");
		_Jumped = false;
		
	}
	
	override public function init()
	{
		
		/* ======================== When Creating ========================= */
		if(_CreateRandomPoint)
		{
			_RandomAngle = asNumber(randomInt(Math.floor(0), Math.floor(360)));
			propertyChanged("_RandomAngle", _RandomAngle);
			_UsedList.push((("" + ("" + (actor.getXCenter() + (_RandomPointRadius * Math.cos(Utils.RAD * (_RandomAngle) * Utils.RAD))))) + ("" + (("" + ",") + ("" + ("" + (actor.getYCenter() + (_RandomPointRadius * Math.sin(Utils.RAD * (_RandomAngle) * Utils.RAD)))))))));
		}
		_UsedList = _PointsList;
		propertyChanged("_UsedList", _UsedList);
		_XPoint = asNumber(("" + _UsedList[Std.int(_PointIndex)]).split(",")[Std.int(0)]);
		propertyChanged("_XPoint", _XPoint);
		_YPoint = asNumber(("" + _UsedList[Std.int(_PointIndex)]).split(",")[Std.int(1)]);
		propertyChanged("_YPoint", _YPoint);
		
		/* ======================== When Updating ========================= */
		addWhenUpdatedListener(null, function(elapsedTime:Float, list:Array<Dynamic>):Void
		{
			if(wrapper.enabled)
			{
				/* Follow */
				if(_Follow)
				{
					if(_FloatAfterPoints)
					{
						/* Float */
						if(!(_PointReached))
						{
							actor.pushInDirection(Utils.DEG * (Math.atan2((_YPoint - actor.getYCenter()), (_XPoint - actor.getXCenter()))), _Acceleration);
						}
						if((actor.getXVelocity() > _TopSpeed))
						{
							actor.setXVelocity(_TopSpeed);
						}
						if((actor.getXVelocity() < -(_TopSpeed)))
						{
							actor.setXVelocity(-(_TopSpeed));
						}
						if((actor.getYVelocity() > _TopSpeed))
						{
							actor.setYVelocity(_TopSpeed);
						}
						if((actor.getYVelocity() < -(_TopSpeed)))
						{
							actor.setYVelocity(-(_TopSpeed));
						}
					}
					else
					{
						if(!(_PointReached))
						{
							/* Walk */
							if(((actor.getXCenter() > (_XPoint - _XRange)) && (actor.getXCenter() < (_XPoint + _XRange))))
							{
								actor.setXVelocity((actor.getXVelocity() * _Slowdown));
							}
							else
							{
								if((actor.getXCenter() >= _XPoint))
								{
									actor.setXVelocity(-(_TopSpeed));
								}
								else
								{
									actor.setXVelocity(_TopSpeed);
								}
							}
							/* Jump */
							if(((actor.getYCenter() - _HeighttoJump) > _YPoint))
							{
								_customEvent_Jump();
							}
							if(((actor.getXCenter() > (_XPoint + _XRange)) && !(tileCollisionAt(Std.int(getTilePosition(1,(actor.getYCenter() + (actor.getHeight())))),Std.int(getTilePosition(0,actor.getXCenter())),0,"" + "-1"))))
							{
								_customEvent_Jump();
							}
							if(((actor.getXCenter() < (_XPoint - _XRange)) && !(tileCollisionAt(Std.int(getTilePosition(1,(actor.getYCenter() + (actor.getHeight())))),Std.int(getTilePosition(0,actor.getXCenter())),0,"" + "-1"))))
							{
								_customEvent_Jump();
							}
							if((((actor.getY() + (actor.getHeight())) < _YPoint) && (Math.abs((actor.getXCenter() - _XPoint)) < _JumpRange)))
							{
								if((actor.getYVelocity() < 0))
								{
									actor.setYVelocity((actor.getYVelocity() * _JumpSlowdown));
								}
							}
						}
					}
					/* Stop Float */
					if((_FloatAfterPoints && _PointReached))
					{
						actor.setXVelocity((actor.getXVelocity() * _Slowdown));
						actor.setYVelocity((actor.getYVelocity() * _Slowdown));
					}
					/* Point Reached */
					if((((actor.getXCenter() >= (_XPoint - _XRange)) && (actor.getXCenter() <= (_XPoint + _XRange))) && ((actor.getYCenter() >= (_YPoint - _YRange)) && (actor.getYCenter() <= (_YPoint + _YRange)))))
					{
						if(!(_PointReached))
						{
							_PointReached = true;
							propertyChanged("_PointReached", _PointReached);
							actor.setXVelocity((actor.getXVelocity() * _Slowdown));
							runLater(1000 * _WaitTime, function(timeTask:TimedTask):Void {
								_PointReached = false;
								propertyChanged("_PointReached", _PointReached);
								if(_Reverse)
								{
									_PointIndex -= 1;
									propertyChanged("_PointIndex", _PointIndex);
								}
								else
								{
									_PointIndex += 1;
									propertyChanged("_PointIndex", _PointIndex);
								}
								if((_PointIndex < 0))
								{
									_Reverse = false;
									propertyChanged("_Reverse", _Reverse);
									_PointIndex += 1;
									propertyChanged("_PointIndex", _PointIndex);
								}
								if((_PointIndex >= _UsedList.length))
								{
									if((_LoopMode == "No Loop"))
									{
										_PointIndex = asNumber(0);
										propertyChanged("_PointIndex", _PointIndex);
										_customEvent_StopFollowPoint();
									}
									if((_LoopMode == "Restart"))
									{
										_PointIndex = asNumber(0);
										propertyChanged("_PointIndex", _PointIndex);
									}
									if((_LoopMode == "Reverse"))
									{
										_Reverse = true;
										propertyChanged("_Reverse", _Reverse);
										_PointIndex -= 1;
										propertyChanged("_PointIndex", _PointIndex);
									}
								}
								/* Set Points */
								_XPoint = asNumber(asNumber(("" + _UsedList[Std.int(_PointIndex)]).split(",")[Std.int(0)]));
								propertyChanged("_XPoint", _XPoint);
								_YPoint = asNumber(asNumber(("" + _UsedList[Std.int(_PointIndex)]).split(",")[Std.int(1)]));
								propertyChanged("_YPoint", _YPoint);
							}, actor);
						}
					}
				}
			}
		});
		
		/* ========================= When Drawing ========================= */
		addWhenDrawingListener(null, function(g:G, x:Float, y:Float, list:Array<Dynamic>):Void
		{
			if(wrapper.enabled)
			{
				if(_Debug)
				{
					g.translateToScreen();
					g.alpha = (100/100);
					g.fillColor = _DebugColor;
					g.strokeColor = _DebugColor;
					g.strokeSize = Std.int(1);
					for(item in cast(_UsedList, Array<Dynamic>))
					{
						g.beginFillPolygon();
						g.addPointToPolygon((asNumber(("" + item).split(",")[Std.int(0)]) - getScreenX()), ((asNumber(("" + item).split(",")[Std.int(1)]) - _YRange) - getScreenY()));
						g.addPointToPolygon(((asNumber(("" + item).split(",")[Std.int(0)]) + _XRange) - getScreenX()), (asNumber(("" + item).split(",")[Std.int(1)]) - getScreenY()));
						g.addPointToPolygon((asNumber(("" + item).split(",")[Std.int(0)]) - getScreenX()), ((asNumber(("" + item).split(",")[Std.int(1)]) + _YRange) - getScreenY()));
						g.addPointToPolygon(((asNumber(("" + item).split(",")[Std.int(0)]) - _XRange) - getScreenX()), (asNumber(("" + item).split(",")[Std.int(1)]) - getScreenY()));
						g.addPointToPolygon((asNumber(("" + item).split(",")[Std.int(0)]) - getScreenX()), ((asNumber(("" + item).split(",")[Std.int(1)]) - _YRange) - getScreenY()));
						g.endDrawingPolygon();
					}
					g.fillColor = _DebugColor2;
					g.strokeColor = _DebugColor2;
					g.beginFillPolygon();
					g.addPointToPolygon((_XPoint - getScreenX()), ((_YPoint - _YRange) - getScreenY()));
					g.addPointToPolygon(((_XPoint + _XRange) - getScreenX()), (_YPoint - getScreenY()));
					g.addPointToPolygon((_XPoint - getScreenX()), ((_YPoint + _YRange) - getScreenY()));
					g.addPointToPolygon(((_XPoint - _XRange) - getScreenX()), (_YPoint - getScreenY()));
					g.addPointToPolygon((_XPoint - getScreenX()), ((_YPoint - _YRange) - getScreenY()));
					g.endDrawingPolygon();
				}
			}
		});
		
		/* ======================== Something Else ======================== */
		addCollisionListener(actor, function(event:Collision, list:Array<Dynamic>):Void
		{
			if(wrapper.enabled)
			{
				if(_Follow)
				{
					if(event.thisCollidedWithTile)
					{
						for(point in event.points)
						{
							if(((Math.abs(Math.round(Engine.toPixelUnits(point.x))) == 10) && (event.thisFromLeft || event.thisFromRight)))
							{
								actor.setYVelocity(-(_JumpForce));
							}
						}
					}
				}
				if(event.thisFromBottom)
				{
					_Jumped = false;
					propertyChanged("_Jumped", _Jumped);
				}
			}
		});
		
	}
	
	override public function forwardMessage(msg:String)
	{
		
	}
}